TIME = "/time"
CREATE_API_KEY = "/create-api-key"
GET_API_KEYS = "/get-api-keys"
TRADE_HISTORY = "/trade-history"
GET_ORDER = "/order/"
OPEN_ORDERS = "/open-orders"
POST_ORDER = "/order"
CANCEL = "/order"
CANCEL_ALL = "/cancel-all"
